"use strict";
const _ = require("lodash");
const jaul = require("jaul");
let settings = require("setmeup").settings;
class Logger {
    static argsCleaner(obj, index) {
        const max = settings.logger.maxDepth;
        if (_.isArray(obj)) {
            for (let i = 0; i < obj.length; i++) {
                if (index >= max) {
                    obj[i] = "...";
                }
                else if (_.isFunction(obj[i])) {
                    obj[i] = "[Function]";
                }
                else {
                    Logger.argsCleaner(obj[i], index + 1);
                }
            }
        }
        else if (_.isObject(obj)) {
            let keys = _.keys(obj);
            for (let key of keys) {
                let value = obj[key];
                try {
                    if (index >= max) {
                        obj[key] = "...";
                    }
                    else if (settings.logger.obfuscateFields && settings.logger.obfuscateFields.indexOf(key) >= 0) {
                        obj[key] = "****";
                    }
                    else if (settings.logger.maskFields && settings.logger.maskFields[key]) {
                        let maskedValue;
                        if (_.isObject(value)) {
                            maskedValue = value.value || value.text || value.data || value.toString();
                        }
                        else {
                            maskedValue = value.toString();
                        }
                        obj[key] = jaul.data.maskString(maskedValue, "*", settings.logger.maskFields[key]);
                    }
                    else if (settings.logger.removeFields && settings.logger.removeFields.indexOf(key) >= 0) {
                        delete obj[key];
                    }
                    else if (_.isArray(value)) {
                        for (let i = 0; i < value.length; i++) {
                            Logger.argsCleaner(value[i], index + 1);
                        }
                    }
                    else if (_.isFunction(value)) {
                        obj[key] = "[Function]";
                    }
                    else if (_.isObject(value)) {
                        Logger.argsCleaner(value, index + 1);
                    }
                }
                catch (ex) {
                    delete obj[key];
                    obj[key] = "[Unreadable]";
                }
            }
        }
    }
    static clean(args) {
        let result = [];
        for (let arg of args) {
            if (_.isError(arg)) {
                result.push(arg.toString());
            }
            else if (_.isObject(arg) || _.isArray(arg)) {
                let cloned = _.cloneDeep(arg);
                Logger.argsCleaner(cloned, 0);
                result.push(cloned);
            }
            else if (_.isFunction(arg)) {
                result.push("[Function]");
            }
            else {
                result.push(arg);
            }
        }
        return result;
    }
}
module.exports = Logger;
